program main
	use MXAPI
	use iso_c_binding

	implicit none

	integer*2 :: r									! return value
	integer*4 :: r32
    integer*4,parameter :: UserCode=1234
	integer*4,allocatable,dimension(:) :: fldData
	integer*2 :: count
    integer*2,parameter :: port=85
	integer*2,parameter :: dngnr=1
	integer*2 :: i

	character(1) :: ch
	integer*4,dimension(0:1) :: plainData
	integer*4,dimension(0:3) :: encKey


	r=Init_MatrixAPI()

	r32=GetVersionAPI()
	print *,"API Version -> Major Version:", hiWord(r32), "Minor Version:", loWord(r32)

	r = Dongle_Count(port)

	print *, "Dongle Count=", r

	if( r <= 0 ) then
		print *, "No Dongle Connected"
		r = Release_MatrixAPI()
		stop
	end if

	!============================================
	!  Acquire Dongle Info
	!============================================

	r32 = Dongle_Model(DngNr, Port)
	print *,"Dongle Model", r32

	r32=Dongle_Version(DngNr, Port)
	print *,"Dongle Version -> Major Version:", hiWord(r32), "Minor Version:", loWord(r32)

	r = Dongle_MemSize(DngNr, Port)
	print *, "MemSize:", r

	! Calculate the Field Count
	count = r/4

	print *, "Field Count:", count


	r32 = Dongle_ReadSerNr(UserCode, DngNr, Port)
	print *, "Dongle Serial Number:", r32

	!============================================
	!  Field Read / Write test
	!============================================

	! allocate fldData big enough to hold all the field data values
	allocate( fldData(count) )


	r = Dongle_ReadData(UserCode, fldData, Count, DngNr, Port)


	if( r>0 ) then
		do i=1, r
			print *, i, fldData(i)
		end do
	end if

	do i=1 ,r
		fldData(i) = 100+i
	end do

	! write 3 fields only
	r = Dongle_WriteData(UserCode, fldData, int(3, 2), DngNr, Port )

	print *, "Dongle_WriteData Result=", r

	deallocate( fldData )

	!============================================
	!  Encrypt / Decrypt test
	!============================================

	! Encrypt Data
	plainData(0)=123666444
	plainData(1)=42343242

	r=Dongle_EncryptData(UserCode, plainData, DngNr, Port)
	print *, plainData

	r=Dongle_DecryptData(UserCode, plainData, DngNr, Port)
	print *, plainData

	! If the dongle has the same XTEA key as the followings, 
 	! Dongle_EncryptData / MxApp_Encrypt,  Dongle_DecryptData / MxApp_Decrypt
	! must output the same results

	encKey(0) = 123456789
	encKey(1) = 234567891
	encKey(2) = 3456789
	encKey(3) = 456789012

	call MxApp_Encrypt(plainData, encKey)
	print *, plainData

	call MxApp_Decrypt(plainData, encKey)
	print *, plainData


	!============================================
	!  Remote API test
	!============================================

	! enable Remote API 
	r=SetConfig_MatrixNet(int(1,2),"c:\matrix\abc.xxxx"//c_null_char)
	print *,r

	print *, "Remote Refresh Time:", GetConfig_MatrixNet(int(1,2))

	r = LogIn_MatrixNet( UserCode, int(1,2), DngNr)
	print *, "LogIn_MatrixNet:", r

	write (*,"(a)",advance="no") "press any key and return to continue:"
	read *, ch

	r = LogOut_MatrixNet( UserCode, int(1,2), DngNr)
	print *, "LogOut_MatrixNet:", r

	!Disable Remote API
	r=SetConfig_MatrixNet(int(0,2),""//c_null_char)


	r = Release_MatrixAPI()

end program main
